@@ -41,6 +41,16 @@ static void check_async_search_results (GtkSourceSearchContext *context,
 					gboolean                forward,
 					gboolean                start_check);
 
+static gchar *
+get_buffer_contents (GtkTextBuffer *buffer)
+{
+	GtkTextIter start;
+	GtkTextIter end;
+
+	gtk_text_buffer_get_bounds (buffer, &start, &end);
+	return gtk_text_iter_get_visible_text (&start, &end);
+}
+
 /* If we are running from the source dir (e.g. during make check)
  * we override the path to read from the data dir.
  */
@@ -877,13 +887,10 @@ test_replace (void)
 	replaced = gtk_source_search_context_replace (context, &start, &end, "bb", 2, NULL);
 	g_assert (replaced);
 
-	gtk_text_buffer_get_start_iter (text_buffer, &start);
-	gtk_text_buffer_get_end_iter (text_buffer, &end);
-
-	contents = gtk_text_iter_get_visible_text (&start, &end);
+	contents = get_buffer_contents (text_buffer);
 	g_assert_cmpstr (contents, ==, "aabb");
-
 	g_free (contents);
+
 	g_object_unref (source_buffer);
 	g_object_unref (settings);
 	g_object_unref (context);
@@ -897,8 +904,6 @@ test_replace_all (void)
 	GtkSourceSearchSettings *settings = gtk_source_search_settings_new ();
 	GtkSourceSearchContext *context = gtk_source_search_context_new (source_buffer, settings);
 	gint nb_replacements;
-	GtkTextIter start;
-	GtkTextIter end;
 	gchar *contents;
 
 	gtk_text_buffer_set_text (text_buffer, "aaaa", -1);
@@ -908,20 +913,17 @@ test_replace_all (void)
 	nb_replacements = gtk_source_search_context_replace_all (context, "bb", 2, NULL);
 	g_assert_cmpint (nb_replacements, ==, 2);
 
-	gtk_text_buffer_get_start_iter (text_buffer, &start);
-	gtk_text_buffer_get_end_iter (text_buffer, &end);
-
-	contents = gtk_text_iter_get_visible_text (&start, &end);
+	contents = get_buffer_contents (text_buffer);
 	g_assert_cmpstr (contents, ==, "bbbb");
-
 	g_free (contents);
+
 	g_object_unref (source_buffer);
 	g_object_unref (settings);
 	g_object_unref (context);
 }
 
 static void
-test_regex (void)
+test_regex_basics (void)
 {
 	GtkSourceBuffer *source_buffer = gtk_source_buffer_new (NULL);
 	GtkTextBuffer *text_buffer = GTK_TEXT_BUFFER (source_buffer);
@@ -962,9 +964,7 @@ test_regex (void)
 	gtk_text_buffer_get_end_iter (text_buffer, &end);
 	gtk_source_search_context_replace (context, &start, &end, "\\2#\\1", -1, NULL);
 
-	gtk_text_buffer_get_start_iter (text_buffer, &start);
-	gtk_text_buffer_get_end_iter (text_buffer, &end);
-	contents = gtk_text_iter_get_visible_text (&start, &end);
+	contents = get_buffer_contents (text_buffer);
 	g_assert_cmpstr (contents, ==, "bb#aa");
 	g_free (contents);
 
@@ -973,9 +973,7 @@ test_regex (void)
 	gtk_text_buffer_set_text (text_buffer, "aa#bb cc#dd", -1);
 	gtk_source_search_context_replace_all (context, "\\2#\\1", -1, NULL);
 
-	gtk_text_buffer_get_start_iter (text_buffer, &start);
-	gtk_text_buffer_get_end_iter (text_buffer, &end);
-	contents = gtk_text_iter_get_visible_text (&start, &end);
+	contents = get_buffer_contents (text_buffer);
 	g_assert_cmpstr (contents, ==, "bb#aa dd#cc");
 	g_free (contents);
 
@@ -994,8 +992,6 @@ test_regex_at_word_boundaries (void)
 	GtkTextIter iter;
 	GtkTextIter match_start;
 	GtkTextIter match_end;
-	GtkTextIter start;
-	GtkTextIter end;
 	gint offset;
 	gchar *content;
 
@@ -1032,9 +1028,7 @@ test_regex_at_word_boundaries (void)
 	gtk_text_buffer_get_end_iter (text_buffer, &match_end);
 	gtk_source_search_context_replace (context, &match_start, &match_end, "bb", -1, NULL);
 
-	gtk_text_buffer_get_start_iter (text_buffer, &start);
-	gtk_text_buffer_get_end_iter (text_buffer, &end);
-	content = gtk_text_iter_get_visible_text (&start, &end);
+	content = get_buffer_contents (text_buffer);
 	g_assert_cmpstr (content, ==, "&bb");
 	g_free (content);
 
@@ -1048,9 +1042,7 @@ test_regex_at_word_boundaries (void)
 	gtk_text_buffer_get_end_iter (text_buffer, &match_end);
 	gtk_source_search_context_replace (context, &match_start, &match_end, "bb", -1, NULL);
 
-	gtk_text_buffer_get_start_iter (text_buffer, &start);
-	gtk_text_buffer_get_end_iter (text_buffer, &end);
-	content = gtk_text_iter_get_visible_text (&start, &end);
+	content = get_buffer_contents (text_buffer);
 	g_assert_cmpstr (content, ==, "–bb");
 	g_free (content);
 
@@ -1059,6 +1051,156 @@ test_regex_at_word_boundaries (void)
 	g_object_unref (context);
 }
 
+static void
+test_regex_look_behind (void)
+{
+	GtkSourceBuffer *source_buffer = gtk_source_buffer_new (NULL);
+	GtkTextBuffer *text_buffer = GTK_TEXT_BUFFER (source_buffer);
+	GtkSourceSearchSettings *settings = gtk_source_search_settings_new ();
+	GtkSourceSearchContext *context = gtk_source_search_context_new (source_buffer, settings);
+	GtkTextIter iter;
+	GtkTextIter match_start;
+	GtkTextIter match_end;
+	gint count;
+	gint pos;
+	gint offset;
+	gboolean found;
+	gchar *contents;
+	GError *error = NULL;
+
+	gtk_text_buffer_set_text (text_buffer, "12\n23\n123\n23\n12", -1);
+
+	gtk_source_search_settings_set_regex_enabled (settings, TRUE);
+	gtk_source_search_settings_set_search_text (settings, "(?<=1)23");
+	flush_queue ();
+
+	/* Occurrences count */
+	count = gtk_source_search_context_get_occurrences_count (context);
+	g_assert_cmpint (count, ==, 1);
+
+	/* Forward search */
+	gtk_text_buffer_get_start_iter (text_buffer, &iter);
+	found = gtk_source_search_context_forward (context, &iter, &match_start, &match_end);
+	g_assert (found);
+
+	offset = gtk_text_iter_get_offset (&match_start);
+	g_assert_cmpint (offset, ==, 7);
+	offset = gtk_text_iter_get_offset (&match_end);
+	g_assert_cmpint (offset, ==, 9);
+
+	/* Backward search */
+	gtk_text_buffer_get_end_iter (text_buffer, &iter);
+	found = gtk_source_search_context_backward (context, &iter, &match_start, &match_end);
+	g_assert (found);
+
+	offset = gtk_text_iter_get_offset (&match_start);
+	g_assert_cmpint (offset, ==, 7);
+	offset = gtk_text_iter_get_offset (&match_end);
+	g_assert_cmpint (offset, ==, 9);
+
+	/* Occurrence position */
+	pos = gtk_source_search_context_get_occurrence_position (context, &match_start, &match_end);
+	g_assert_cmpint (pos, ==, 1);
+
+	/* Replace */
+	gtk_source_search_context_replace (context, &match_start, &match_end, "R", -1, &error);
+	g_assert_no_error (error);
+
+	contents = get_buffer_contents (text_buffer);
+	g_assert_cmpstr (contents, ==, "12\n23\n1R\n23\n12");
+	g_free (contents);
+
+	/* Replace all */
+	gtk_text_buffer_set_text (text_buffer, "12\n23\n123 123\n23\n12", -1);
+	flush_queue ();
+
+	gtk_source_search_context_replace_all (context, "R", -1, &error);
+	g_assert_no_error (error);
+
+	contents = get_buffer_contents (text_buffer);
+	g_assert_cmpstr (contents, ==, "12\n23\n1R 1R\n23\n12");
+	g_free (contents);
+
+	g_object_unref (source_buffer);
+	g_object_unref (settings);
+	g_object_unref (context);
+}
+
+static void
+test_regex_look_ahead (void)
+{
+	GtkSourceBuffer *source_buffer = gtk_source_buffer_new (NULL);
+	GtkTextBuffer *text_buffer = GTK_TEXT_BUFFER (source_buffer);
+	GtkSourceSearchSettings *settings = gtk_source_search_settings_new ();
+	GtkSourceSearchContext *context = gtk_source_search_context_new (source_buffer, settings);
+	GtkTextIter iter;
+	GtkTextIter match_start;
+	GtkTextIter match_end;
+	gint count;
+	gint pos;
+	gint offset;
+	gboolean found;
+	gchar *contents;
+	GError *error = NULL;
+
+	gtk_text_buffer_set_text (text_buffer, "12\n23\n123\n23\n12", -1);
+
+	gtk_source_search_settings_set_regex_enabled (settings, TRUE);
+	gtk_source_search_settings_set_search_text (settings, "12(?=3)");
+	flush_queue ();
+
+	/* Occurrences count */
+	count = gtk_source_search_context_get_occurrences_count (context);
+	g_assert_cmpint (count, ==, 1);
+
+	/* Forward search */
+	gtk_text_buffer_get_start_iter (text_buffer, &iter);
+	found = gtk_source_search_context_forward (context, &iter, &match_start, &match_end);
+	g_assert (found);
+
+	offset = gtk_text_iter_get_offset (&match_start);
+	g_assert_cmpint (offset, ==, 6);
+	offset = gtk_text_iter_get_offset (&match_end);
+	g_assert_cmpint (offset, ==, 8);
+
+	/* Backward search */
+	gtk_text_buffer_get_end_iter (text_buffer, &iter);
+	found = gtk_source_search_context_backward (context, &iter, &match_start, &match_end);
+	g_assert (found);
+
+	offset = gtk_text_iter_get_offset (&match_start);
+	g_assert_cmpint (offset, ==, 6);
+	offset = gtk_text_iter_get_offset (&match_end);
+	g_assert_cmpint (offset, ==, 8);
+
+	/* Occurrence position */
+	pos = gtk_source_search_context_get_occurrence_position (context, &match_start, &match_end);
+	g_assert_cmpint (pos, ==, 1);
+
+	/* Replace */
+	gtk_source_search_context_replace (context, &match_start, &match_end, "R", -1, &error);
+	g_assert_no_error (error);
+
+	contents = get_buffer_contents (text_buffer);
+	g_assert_cmpstr (contents, ==, "12\n23\nR3\n23\n12");
+	g_free (contents);
+
+	/* Replace all */
+	gtk_text_buffer_set_text (text_buffer, "12\n23\n123 123\n23\n12", -1);
+	flush_queue ();
+
+	gtk_source_search_context_replace_all (context, "R", -1, &error);
+	g_assert_no_error (error);
+
+	contents = get_buffer_contents (text_buffer);
+	g_assert_cmpstr (contents, ==, "12\n23\nR3 R3\n23\n12");
+	g_free (contents);
+
+	g_object_unref (source_buffer);
+	g_object_unref (settings);
+	g_object_unref (context);
+}
+
 static void
 test_destroy_buffer_during_search (void)
 {
@@ -1105,9 +1247,11 @@ main (int argc, char **argv)
 	g_test_add_func ("/Search/get-search-text", test_get_search_text);
 	g_test_add_func ("/Search/occurrence-position", test_occurrence_position);
 	g_test_add_func ("/Search/replace", test_replace);
-	g_test_add_func ("/Search/replace", test_replace_all);
-	g_test_add_func ("/Search/regex", test_regex);
-	g_test_add_func ("/Search/regex-at-word-boundaries", test_regex_at_word_boundaries);
+	g_test_add_func ("/Search/replace-all", test_replace_all);
+	g_test_add_func ("/Search/regex/basics", test_regex_basics);
+	g_test_add_func ("/Search/regex/at-word-boundaries", test_regex_at_word_boundaries);
+	g_test_add_func ("/Search/regex/look-behind", test_regex_look_behind);
+	g_test_add_func ("/Search/regex/look-ahead", test_regex_look_ahead);
 	g_test_add_func ("/Search/destroy-buffer-during-search", test_destroy_buffer_during_search);
 
 	return g_test_run ();
